@@ -1,7 +1,7 @@
 #!/bin/bash
 
-# rust create_release v0.5.7
-# 2024-08-02
+# rust create_release v0.6.1
+# 2024-10-21
 
 STAR_LINE='****************************************'
 CWD=$(pwd)
@@ -24,20 +24,25 @@ if ! [ -x "$(command -v dialog)" ]; then
 fi
 
 # $1 string - question to ask
+# $1 question to ask
+# Ask a yes no question, only accepts `y` or `n` as a valid answer, returns 0 for yes, 1 for no
 ask_yn() {
-	printf "%b%s? [y/N]:%b " "${GREEN}" "$1" "${RESET}"
-}
-
-# return user input
-user_input() {
-	read -r data
-	echo "$data"
+	while true; do
+		printf "\n%b%s? [y/N]:%b " "${GREEN}" "$1" "${RESET}"
+		read -r answer
+		if [[ "$answer" == "y" ]]; then
+			return 0
+		elif [[ "$answer" == "n" ]]; then
+			return 1
+		else
+			echo -e "${RED}\nPlease enter 'y' or 'n'${RESET}"
+		fi
+	done
 }
 
 # ask continue, or quit
 ask_continue() {
-	ask_yn "continue"
-	if [[ ! "$(user_input)" =~ ^y$ ]]; then
+	if ! ask_yn "continue"; then
 		exit
 	fi
 }
@@ -93,12 +98,13 @@ ask_changelog_update() {
 	RELEASE_BODY_TEXT=$(sed '/# <a href=/Q' CHANGELOG.md)
 	printf "%s" "$RELEASE_BODY_TEXT"
 	printf "\n%s\n" "${STAR_LINE}"
-	ask_yn "accept release body"
-	if [[ "$(user_input)" =~ ^y$ ]]; then
+
+	if ask_yn "accept release body"; then
 		update_release_body_and_changelog "$RELEASE_BODY_TEXT"
 	else
 		exit
 	fi
+
 }
 
 # Edit the release-body to include new lines from changelog
@@ -122,6 +128,10 @@ update_release_body_and_changelog() {
 	# Update changelog to add links to closed issues
 	# "closes #1" -> "closes [#1](https:/www.../issues/1)""
 	sed -i -r -E "s=closes \#([0-9]+)=closes [#\1](${GIT_REPO_URL}/issues/\1)=g" CHANGELOG.md
+
+	# Update changelog to add links to merged PR's
+	# "merges #1" -> "merges [#1](https:/www.../pull/1)""
+	sed -i -r -E "s=merges \#([0-9]+)=merges [#\1](${GIT_REPO_URL}/pull/\1)=g" CHANGELOG.md
 }
 
 # update version in cargo.toml, to match selected current version
