@@ -1,16 +1,13 @@
-use crate::app_data::DockerControls;
+use crate::app_data::DockerCommand;
 use std::fmt;
 
 /// app errors to set in global state
-#[allow(unused)]
 #[derive(Debug, Clone, Copy)]
 pub enum AppError {
-    DockerCommand(DockerControls),
+    DockerCommand(DockerCommand),
     DockerExec,
     DockerLogs,
     DockerConnect,
-    DockerInterval,
-    InputPoll,
     MouseCapture(bool),
     Terminal,
 }
@@ -23,8 +20,6 @@ impl fmt::Display for AppError {
             Self::DockerExec => write!(f, "Unable to exec into container"),
             Self::DockerLogs => write!(f, "Unable to save logs"),
             Self::DockerConnect => write!(f, "Unable to access docker daemon"),
-            Self::DockerInterval => write!(f, "Docker update interval needs to be greater than 0"),
-            Self::InputPoll => write!(f, "Unable to poll user input"),
             Self::MouseCapture(x) => {
                 let reason = if *x { "en" } else { "dis" };
                 write!(f, "Unable to {reason}able mouse capture")
